@@ -1,21 +1,21 @@
 # coding=utf-8
 """
-adminchannel.py - Sopel Channel Admin Module
+adminchannel.py - Sopel Channel Admin Plugin
 Copyright 2010-2011, Michael Yanovich, Alek Rollyson, and Elsie Powell
 Copyright © 2012, Elad Alfassa <elad@fedoraproject.org>
 Licensed under the Eiffel Forum License 2.
 
 https://sopel.chat
 """
-from __future__ import unicode_literals, absolute_import, print_function, division
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 import re
 
-from sopel import formatting
-from sopel.module import (
-    commands, example, priority, OP, HALFOP, require_privilege, require_chanmsg
-)
-from sopel.tools import Identifier
+from sopel import formatting, plugin, tools
+
+
+ERROR_MESSAGE_NOT_OP = "I'm not a channel operator!"
+ERROR_MESSAGE_NO_PRIV = "You are not a channel operator."
 
 
 def default_mask(trigger):
@@ -27,16 +27,15 @@ def default_mask(trigger):
     return '{} {} {} {}'.format(welcome, chan, topic_, arg)
 
 
-@require_chanmsg
-@require_privilege(OP, 'You are not a channel operator.')
-@commands('op')
+@plugin.require_chanmsg
+@plugin.require_privilege(plugin.OP, ERROR_MESSAGE_NO_PRIV, reply=True)
+@plugin.require_bot_privilege(plugin.OP, ERROR_MESSAGE_NOT_OP, reply=True)
+@plugin.command('op')
 def op(bot, trigger):
     """
     Command to op users in a room. If no nick is given,
     Sopel will op the nick who sent the command
     """
-    if bot.channels[trigger.sender].privileges[bot.nick] < OP:
-        return bot.reply("I'm not a channel operator!")
     nick = trigger.group(2)
     channel = trigger.sender
     if not nick:
@@ -44,16 +43,15 @@ def op(bot, trigger):
     bot.write(['MODE', channel, "+o", nick])
 
 
-@require_chanmsg
-@require_privilege(OP, 'You are not a channel operator.')
-@commands('deop')
+@plugin.require_chanmsg
+@plugin.require_privilege(plugin.OP, ERROR_MESSAGE_NO_PRIV, reply=True)
+@plugin.require_bot_privilege(plugin.OP, ERROR_MESSAGE_NOT_OP, reply=True)
+@plugin.command('deop')
 def deop(bot, trigger):
     """
     Command to deop users in a room. If no nick is given,
     Sopel will deop the nick who sent the command
     """
-    if bot.channels[trigger.sender].privileges[bot.nick] < OP:
-        return bot.reply("I'm not a channel operator!")
     nick = trigger.group(2)
     channel = trigger.sender
     if not nick:
@@ -61,16 +59,15 @@ def deop(bot, trigger):
     bot.write(['MODE', channel, "-o", nick])
 
 
-@require_chanmsg
-@require_privilege(OP, 'You are not a channel operator.')
-@commands('voice')
+@plugin.require_chanmsg
+@plugin.require_privilege(plugin.OP, ERROR_MESSAGE_NO_PRIV, reply=True)
+@plugin.require_bot_privilege(plugin.HALFOP, ERROR_MESSAGE_NOT_OP, reply=True)
+@plugin.command('voice')
 def voice(bot, trigger):
     """
     Command to voice users in a room. If no nick is given,
     Sopel will voice the nick who sent the command
     """
-    if bot.channels[trigger.sender].privileges[bot.nick] < HALFOP:
-        return bot.reply("I'm not a channel operator!")
     nick = trigger.group(2)
     channel = trigger.sender
     if not nick:
@@ -78,16 +75,15 @@ def voice(bot, trigger):
     bot.write(['MODE', channel, "+v", nick])
 
 
-@require_chanmsg
-@require_privilege(OP, 'You are not a channel operator.')
-@commands('devoice')
+@plugin.require_chanmsg
+@plugin.require_privilege(plugin.OP, ERROR_MESSAGE_NO_PRIV, reply=True)
+@plugin.require_bot_privilege(plugin.HALFOP, ERROR_MESSAGE_NOT_OP, reply=True)
+@plugin.command('devoice')
 def devoice(bot, trigger):
     """
     Command to devoice users in a room. If no nick is given,
     Sopel will devoice the nick who sent the command
     """
-    if bot.channels[trigger.sender].privileges[bot.nick] < HALFOP:
-        return bot.reply("I'm not a channel operator!")
     nick = trigger.group(2)
     channel = trigger.sender
     if not nick:
@@ -95,19 +91,18 @@ def devoice(bot, trigger):
     bot.write(['MODE', channel, "-v", nick])
 
 
-@require_chanmsg
-@require_privilege(OP, 'You are not a channel operator.')
-@commands('kick')
-@priority('high')
+@plugin.require_chanmsg
+@plugin.require_privilege(plugin.OP, ERROR_MESSAGE_NO_PRIV, reply=True)
+@plugin.require_bot_privilege(plugin.HALFOP, ERROR_MESSAGE_NOT_OP, reply=True)
+@plugin.command('kick')
+@plugin.priority('high')
 def kick(bot, trigger):
     """Kick a user from the channel."""
-    if bot.channels[trigger.sender].privileges[bot.nick] < HALFOP:
-        return bot.reply("I'm not a channel operator!")
     text = trigger.group().split()
     argc = len(text)
     if argc < 2:
         return
-    opt = Identifier(text[1])
+    opt = tools.Identifier(text[1])
     nick = opt
     channel = trigger.sender
     reasonidx = 2
@@ -147,22 +142,21 @@ def configureHostMask(mask):
     return ''
 
 
-@require_chanmsg
-@require_privilege(OP, 'You are not a channel operator.')
-@commands('ban')
-@priority('high')
+@plugin.require_chanmsg
+@plugin.require_privilege(plugin.OP, ERROR_MESSAGE_NO_PRIV, reply=True)
+@plugin.require_bot_privilege(plugin.HALFOP, ERROR_MESSAGE_NOT_OP, reply=True)
+@plugin.command('ban')
+@plugin.priority('high')
 def ban(bot, trigger):
     """Ban a user from the channel
 
     The bot must be a channel operator for this command to work.
     """
-    if bot.channels[trigger.sender].privileges[bot.nick] < HALFOP:
-        return bot.reply("I'm not a channel operator!")
     text = trigger.group().split()
     argc = len(text)
     if argc < 2:
         return
-    opt = Identifier(text[1])
+    opt = tools.Identifier(text[1])
     banmask = opt
     channel = trigger.sender
     if not opt.is_nick():
@@ -176,21 +170,20 @@ def ban(bot, trigger):
     bot.write(['MODE', channel, '+b', banmask])
 
 
-@require_chanmsg
-@require_privilege(OP, 'You are not a channel operator.')
-@commands('unban')
+@plugin.require_chanmsg
+@plugin.require_privilege(plugin.OP, ERROR_MESSAGE_NO_PRIV, reply=True)
+@plugin.require_bot_privilege(plugin.HALFOP, ERROR_MESSAGE_NOT_OP, reply=True)
+@plugin.command('unban')
 def unban(bot, trigger):
     """Unban a user from the channel
 
     The bot must be a channel operator for this command to work.
     """
-    if bot.channels[trigger.sender].privileges[bot.nick] < HALFOP:
-        return bot.reply("I'm not a channel operator!")
     text = trigger.group().split()
     argc = len(text)
     if argc < 2:
         return
-    opt = Identifier(text[1])
+    opt = tools.Identifier(text[1])
     banmask = opt
     channel = trigger.sender
     if not opt.is_nick():
@@ -204,21 +197,20 @@ def unban(bot, trigger):
     bot.write(['MODE', channel, '-b', banmask])
 
 
-@require_chanmsg
-@require_privilege(OP, 'You are not a channel operator.')
-@commands('quiet')
+@plugin.require_chanmsg
+@plugin.require_privilege(plugin.OP, ERROR_MESSAGE_NO_PRIV, reply=True)
+@plugin.require_bot_privilege(plugin.OP, ERROR_MESSAGE_NOT_OP, reply=True)
+@plugin.command('quiet')
 def quiet(bot, trigger):
     """Quiet a user
 
     The bot must be a channel operator for this command to work.
     """
-    if bot.channels[trigger.sender].privileges[bot.nick] < OP:
-        return bot.reply("I'm not a channel operator!")
     text = trigger.group().split()
     argc = len(text)
     if argc < 2:
         return
-    opt = Identifier(text[1])
+    opt = tools.Identifier(text[1])
     quietmask = opt
     channel = trigger.sender
     if not opt.is_nick():
@@ -232,21 +224,20 @@ def quiet(bot, trigger):
     bot.write(['MODE', channel, '+q', quietmask])
 
 
-@require_chanmsg
-@require_privilege(OP, 'You are not a channel operator.')
-@commands('unquiet')
+@plugin.require_chanmsg
+@plugin.require_privilege(plugin.OP, ERROR_MESSAGE_NO_PRIV, reply=True)
+@plugin.require_bot_privilege(plugin.OP, ERROR_MESSAGE_NOT_OP, reply=True)
+@plugin.command('unquiet')
 def unquiet(bot, trigger):
     """Unquiet a user
 
     The bot must be a channel operator for this command to work.
     """
-    if bot.channels[trigger.sender].privileges[bot.nick] < OP:
-        return bot.reply("I'm not a channel operator!")
     text = trigger.group().split()
     argc = len(text)
     if argc < 2:
         return
-    opt = Identifier(text[1])
+    opt = tools.Identifier(text[1])
     quietmask = opt
     channel = trigger.sender
     if not opt.is_nick():
@@ -260,23 +251,22 @@ def unquiet(bot, trigger):
     bot.write(['MODE', channel, '-q', quietmask])
 
 
-@require_chanmsg
-@require_privilege(OP, 'You are not a channel operator.')
-@commands('kickban', 'kb')
-@example('.kickban [#chan] user1 user!*@* get out of here')
-@priority('high')
+@plugin.require_chanmsg
+@plugin.require_privilege(plugin.OP, ERROR_MESSAGE_NO_PRIV, reply=True)
+@plugin.require_bot_privilege(plugin.OP, ERROR_MESSAGE_NOT_OP, reply=True)
+@plugin.command('kickban', 'kb')
+@plugin.example('.kickban [#chan] user1 user!*@* get out of here')
+@plugin.priority('high')
 def kickban(bot, trigger):
     """Kick and ban a user from the channel
 
     The bot must be a channel operator for this command to work.
     """
-    if bot.channels[trigger.sender].privileges[bot.nick] < HALFOP:
-        return bot.reply("I'm not a channel operator!")
     text = trigger.group().split()
     argc = len(text)
     if argc < 4:
         return
-    opt = Identifier(text[1])
+    opt = tools.Identifier(text[1])
     nick = opt
     mask = text[2]
     channel = trigger.sender
@@ -296,16 +286,20 @@ def kickban(bot, trigger):
     bot.kick(nick, channel, reason)
 
 
-@require_chanmsg
-@require_privilege(OP, 'You are not a channel operator.')
-@commands('topic')
+@plugin.require_chanmsg
+@plugin.command('topic')
 def topic(bot, trigger):
     """Change the channel topic
 
-    The bot must be a channel operator for this command to work.
+    The bot must be a channel operator for this command to work in +t channels.
     """
-    if bot.channels[trigger.sender].privileges[bot.nick] < HALFOP:
-        return bot.reply("I'm not a channel operator!")
+    mode_t = bot.channels[trigger.sender].modes.get("t", False)
+    if mode_t and not bot.has_channel_privilege(trigger.sender, plugin.HALFOP):
+        bot.reply(ERROR_MESSAGE_NOT_OP)
+        return
+    if mode_t and bot.channels[trigger.sender].privileges[trigger.nick] < plugin.HALFOP:
+        bot.reply(ERROR_MESSAGE_NO_PRIV)
+        return
     if not trigger.group(2):
         return
     channel = trigger.sender.lower()
@@ -322,17 +316,28 @@ def topic(bot, trigger):
         args = top.split('~', narg)
 
     if len(args) != narg:
-        message = "Not enough arguments. You gave {}, it requires {}.".format(
+        message = "Not enough arguments. You gave {}; it requires {}.".format(
             len(args), narg)
-        return bot.say(message)
+        bot.reply(message)
+        return
     topic = mask.format(*args)
 
+    topiclen = getattr(bot.isupport, 'TOPICLEN', None)
+    if topiclen is not None:
+        my_len = len(topic.encode('utf-8'))
+        if my_len > topiclen:
+            bot.reply(
+                "Formatted topic is too long ({} bytes); "
+                "the server limit is {} bytes."
+                .format(my_len, topiclen))
+            return
+
     bot.write(('TOPIC', channel + ' :' + topic))
 
 
-@require_chanmsg
-@require_privilege(OP, 'You are not a channel operator.')
-@commands('tmask')
+@plugin.require_chanmsg
+@plugin.require_privilege(plugin.OP, ERROR_MESSAGE_NO_PRIV)
+@plugin.command('tmask')
 def set_mask(bot, trigger):
     """Set the topic mask to use for the current channel
 
@@ -341,12 +346,17 @@ def set_mask(bot, trigger):
     This mask is used when running the 'topic' command.
     """
     bot.db.set_channel_value(trigger.sender, 'topic_mask', trigger.group(2))
-    bot.say("Gotcha, " + trigger.nick)
+    message = (
+        'Topic mask set. '
+        'Use `{prefix}topic <args>` to set topic '
+        'and `{prefix}showmask` to see current mask.'
+    ).format(prefix=bot.settings.core.help_prefix)
+    bot.reply(message)
 
 
-@require_chanmsg
-@require_privilege(OP, 'You are not a channel operator.')
-@commands('showmask')
+@plugin.require_chanmsg
+@plugin.require_privilege(plugin.OP, ERROR_MESSAGE_NO_PRIV)
+@plugin.command('showmask')
 def show_mask(bot, trigger):
     """Show the topic mask for the current channel."""
     bot.say(bot.db.get_channel_value(trigger.sender, 'topic_mask', default_mask(trigger)))
