@@ -13,14 +13,20 @@ from collections import defaultdict
 import io  # don't use `codecs` for loading the DB; it will split lines on some IRC formatting
 import logging
 import os
+import sys
 import threading
 import time
+import unicodedata
 
-from sopel import plugin, tools
+from sopel import formatting, plugin, tools
 from sopel.config import types
 from sopel.tools.time import format_time, get_timezone
 
 
+if sys.version_info.major >= 3:
+    unicode = str
+
+
 LOGGER = logging.getLogger(__name__)
 
 
@@ -130,6 +136,43 @@ def shutdown(bot):
             pass
 
 
+def _format_safe_lstrip(text):
+    """``str.lstrip()`` but without eating IRC formatting.
+
+    :param str text: text to clean
+    :rtype: str
+    :raises TypeError: if the passed ``text`` is not a string
+
+    Stolen and tweaked from the ``choose`` plugin's ``_format_safe()``
+    function by the person who wrote it.
+    """
+    if not isinstance(text, unicode):
+        raise TypeError("A string is required.")
+    elif not text:
+        # unnecessary optimization
+        return ''
+
+    start = 0
+
+    # strip left
+    pos = 0
+    while pos < len(text):
+        is_whitespace = unicodedata.category(text[pos]) == 'Zs'
+        is_non_printing = (
+            text[pos] in formatting.CONTROL_NON_PRINTING and
+            text[pos] not in formatting.CONTROL_FORMATTING
+        )
+        if not is_whitespace and not is_non_printing:
+            start = pos
+            break
+        pos += 1
+    else:
+        # skipped everything; string is all whitespace
+        return ''
+
+    return text[start:]
+
+
 @plugin.command('tell', 'ask')
 @plugin.nickname_command('tell', 'ask')
 @plugin.example('$nickname, tell dgw he broke something again.')
@@ -143,7 +186,7 @@ def f_remind(bot, trigger):
         return
 
     tellee = trigger.group(3).rstrip('.,:;')
-    msg = trigger.group(2).lstrip(tellee).lstrip()
+    msg = _format_safe_lstrip(trigger.group(2).split(' ', 1)[1])
 
     if not msg:
         bot.reply("%s %s what?" % (verb, tellee))
